from enum import Enum


class TipoLogradouro(Enum):
    AREA = "A"
    ACESSO = "AC"
    ACAMPAMENTO = "ACA"
    ACESSO_LOCAL = "ACL"
    ADRO = "AD"
    AREA_ESPECIAL = "AE"
    AEROPORTO = "AER"
    ALAMEDA = "AL"
    AVENIDA_MARGINAL_DIREITA = "AMD"
    AVENIDA_MARGINAL_ESQUERDA = "AME"
    ANEL_VIARIO = "AN"
    ANTIGA_ESTRADA = "ANT"
    ARTERIA = "ART"
    ALTO = "AT"
    ATALHO = "ATL"
    AREA_VERDE = "A V"
    AVENIDA = "AV"
    AVENIDA_CONTORNO = "AVC"
    AVENIDA_MARGINAL = "AVM"
    AVENIDA_VELHA = "AVV"
    BALNEARIO = "BAL"
    BECO = "BC"
    BURACO = "BCO"
    BELVEDERE = "BEL"
    BLOCO = "BL"
    BALO = "BLO"
    BLOCOS = "BLS"
    BULEVAR = "BLV"
    BOSQUE = "BSQ"
    BOULEVARD = "BVD"
    BAIXA = "BX"
    CAIS = "C"
    CALCADA = "CAL"
    CAMINHO = "CAM"
    CANAL = "CAN"
    CHACARA = "CH"
    CHAPADAO = "CHA"
    CICLOVIA = "CIC"
    CIRCULAR = "CIR"
    CONJUNTO = "CJ"
    CONJUNTO_MUTIRAO = "CJM"
    COMPLEXO_VIARIO = "CMP"
    COLONIA = "COL"
    COMUNIDADE = "COM"
    CONDOMINIO = "CON"
    CORREDOR = "COR"
    CAMPO = "CPO"
    CORREGO = "CRG"
    CONTORNO = "CTN"
    DESCIDA = "DSC"
    DESVIO = "DSV"
    DISTRITO = "DT"
    ENTRE_BLOCO = "EB"
    ESTRADA_INTERMUNICIPAL = "EIM"
    ENSEADA = "ENS"
    ENTRADA_PARTICULAR = "ENT"
    ENTRE_QUADRA = "EQ"
    ESCADA = "ESC"
    ESCADARIA = "ESD"
    ESTRADA_ESTADUAL = "ESE"
    ESTRADA_VICINAL = "ESI"
    ESTRADA_DE_LIGACAO = "ESL"
    ESTRADA_MUNICIPAL = "ESM"
    ESPLANADA = "ESP"
    ESTRADA_DE_SERVIDAO = "ESS"
    ESTRADA = "EST"
    ESTRADA_VELHA = "ESV"
    ESTRADA_ANTIGA = "ETA"
    ESTACAO = "ETC"
    ESTADIO = "ETD"
    ESTANCIA = "ETN"
    ESTRADA_PARTICULAR = "ETP"
    ESTACIONAMENTO = "ETT"
    EVANGELICA = "EVA"
    ELEVADA = "EVD"
    EIXO_INDUSTRIAL = "EX"
    FAVELA = "FAV"
    FAZENDA = "FAZ"
    FERROVIA = "FER"
    FONTE = "FNT"
    FEIRA = "FRA"
    FORTE = "FTE"
    GALERIA = "GAL"
    GRANJA = "GJA"
    NUCLEO_HABITACIONAL = "HAB"
    ILHA = "IA"
    INDETERMINADO = "IND"
    ILHOTA = "IOA"
    JARDIM = "JD"
    JARDINETE = "JDE"
    LADEIRA = "LD"
    LAGOA = "LGA"
    LAGO = "LGO"
    LOTEAMENTO = "LOT"
    LARGO = "LRG"
    LOTE = "LT"
    MERCADO = "MER"
    MARINA = "MNA"
    MODULO = "MOD"
    PROJECAO = "MRG"
    MORRO = "MRO"
    MONTE = "MTE"
    NUCLEO = "NUC"
    NUCLEO_RURAL = "NUR"
    OUTEIRO = "OUT"
    PARALELA = "PAR"
    PASSEIO = "PAS"
    PATIO = "PAT"
    PRACA = "PC"
    PRACA_DE_ESPORTES = "PCE"
    PARADA = "PDA"
    PARADOURO = "PDO"
    PONTA = "PNT"
    PRAIA = "PR"
    PROLONGAMENTO = "PRL"
    PARQUE_MUNICIPAL = "PRM"
    PARQUE = "PRQ"
    PARQUE_RESIDENCIAL = "PRR"
    PASSARELA = "PSA"
    PASSAGEM = "PSG"
    PASSAGEM_DE_PEDESTRE = "PSP"
    PASSAGEM_SUBTERRANEA = "PSS"
    PONTE = "PTE"
    PORTO = "PTO"
    QUADRA = "Q"
    QUINTA = "QTA"
    QUINTAS = "QTS"
    RUA = "R"
    RUA_INTEGRACAO = "R I"
    RUA_DE_LIGACAO = "R L"
    RUA_PARTICULAR = "R P"
    RUA_VELHA = "R V"
    RAMAL = "RAM"
    RECREIO = "RCR"
    RECANTO = "REC"
    RETIRO = "RER"
    RESIDENCIAL = "RES"
    RETA = "RET"
    RUELA = "RLA"
    RAMPA = "RMP"
    RODO_ANEL = "ROA"
    RODOVIA = "ROD"
    ROTULA = "ROT"
    RUA_DE_PEDESTRE = "RPE"
    MARGEM = "RPR"
    RETORNO = "RTN"
    ROTATORIA = "RTT"
    SEGUNDA_AVENIDA = "SEG"
    SITIO = "SIT"
    SERVIDAO = "SRV"
    SETOR = "ST"
    SUBIDA = "SUB"
    TRINCHEIRA = "TCH"
    TERMINAL = "TER"
    TRECHO = "TR"
    TREVO = "TRV"
    TUNEL = "TUN"
    TRAVESSA = "TV"
    TRAVESSA_PARTICULAR = "TVP"
    TRAVESSA_VELHA = "TVV"
    UNIDADE = "UNI"
    VIA = "V"
    VIA_COLETORA = "V C"
    VIA_LOCAL = "V L"
    VIA_DE_ACESSO = "VAC"
    VALA = "VAL"
    VIA_COSTEIRA = "VCO"
    VIADUTO = "VD"
    VIA_EXPRESSA = "V-E"
    VEREDA = "VER"
    VIA_ELEVADO = "VEV"
    VILA = "VL"
    VIELA = "VLA"
    VALE = "VLE"
    VIA_LITORANEA = "VLT"
    VIA_DE_PEDESTRE = "VPE"
    VARIANTE = "VRT"
    ZIGUE_ZAGUE = "ZIG"
